<?php
/**
 * Cost of Goods for WooCommerce - Compatibility with WooCommerce Food.
 *
 * @link    https://docs.wpsyncsheets.com/wpssw-introduction/
 *
 * @version 3.2.4
 * @since   3.2.4
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro_Compatibility_WPSyncSheets' ) ) {

	class Alg_WC_Cost_of_Goods_Pro_Compatibility_WPSyncSheets {
		/**
		 * Header list.
		 *
		 * @version 3.2.4
		 * @since   3.2.4
		 *
		 * @var array $wpssw_headers
		 */
		public static $wpssw_headers = array();

		/**
		 * Class Contructor.
		 *
		 * @version 3.2.4
		 * @since   3.2.4
		 *
		 */
		public function init() {
			$this->prepare_headers();
			add_filter( 'wpsyncsheets_order_headers', array( static::class, 'get_header_list' ), 10, 1 );
		}

		/**
		 * Prepare Header List.
		 *
		 * @version 3.2.4
		 * @since   3.2.4
		 */
		protected function prepare_headers() {
			if ( 'yes' === get_option( 'alg_wc_cog_comp_wpsyncsheets_export_order_cost', 'no' ) ) {
				self::$wpssw_headers['_alg_wc_cog_order_cost'] = __( 'Cost', 'cost-of-goods-for-woocommerce' );
			}

			if ( 'yes' === get_option( 'alg_wc_cog_comp_wpsyncsheets_export_order_profit', 'no' ) ) {
				self::$wpssw_headers['_alg_wc_cog_order_profit'] = __( 'Profit', 'cost-of-goods-for-woocommerce' );
			}
		}

		/**
		 * Get Header List.
		 *
		 * @version 3.2.4
		 * @since   3.2.4
		 *
		 * @param   array  $headers
		 */
		public static function get_header_list( $headers = array() ) {
			if ( ! empty( self::$wpssw_headers ) ) {
				$headers['Alg_WC_Cost_of_Goods_Pro_Compatibility_WPSyncSheets'] = self::$wpssw_headers;
			}

			return $headers;
		}

		/**
		 * Get Value for given header name.
		 *
		 * @version 3.2.4
		 * @since   3.2.4
		 *
		 * @param   string  $wpssw_headers_name  Header name.
		 * @param   object  $wpssw_order         order object.
		 */
		public static function get_value( $wpssw_headers_name, $wpssw_order ) {
			return self::prepare_value( $wpssw_headers_name, $wpssw_order );
		}

		/**
		 * Prepare Value for given header name.
		 *
		 * @version 3.2.4
		 * @since   3.2.4
		 *
		 * @param   string  $wpssw_headers_name  Header name.
		 * @param   object  $wpssw_order         order object.
		 */
		public static function prepare_value( $wpssw_headers_name, $wpssw_order ) {
			if (
				is_a( $wpssw_order, 'WC_Order' ) &&
				false !== ( $key = array_search( $wpssw_headers_name, self::$wpssw_headers ) ) &&
				in_array( $key, array( '_alg_wc_cog_order_cost', '_alg_wc_cog_order_profit' ) )
			) {
				$wpssw_value[] = $wpssw_order->get_meta( $key );
			}

			return $wpssw_value;
		}
	}
}