<?php
/**
 * Cost of Goods for WooCommerce - Pro - Compatibility.
 *
 * @version 3.2.4
 * @since   2.5.7
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro_Compatibility' ) ) {

	class Alg_WC_Cost_of_Goods_Pro_Compatibility {

		/**
		 * init.
		 *
		 * @version 3.1.9
		 * @since   2.5.7
		 *
		 */
		function init(){
			// WP All Import.
			add_filter( 'pmxi_custom_field', array( $this, 'fix_cost_from_wp_all_import' ), 10, 6 );

			// Compatibility with WPC Product Bundles for WooCommerce.
			add_filter( 'alg_wc_cog_stock_report_args', array( $this, 'exclude_wpc_bundle_from_stock_report' ) );
			add_filter( 'woocommerce_reports_get_order_report_query', array( $this, 'exclude_wpc_bundle_from_orders_report' ) );

			// Compatibility with Openpos - WooCommerce Point Of Sale(POS).
			add_filter( 'woocommerce_reports_get_order_report_query', array( $this, 'setup_openpos_order_reports_query' ) );

			// Metorik.
			add_action( 'added_post_meta', array( $this, 'metorik_sync_cost' ), 10, 4 );
			add_action( 'updated_post_meta', array( $this, 'metorik_sync_cost' ), 10, 4 );
			add_action( 'deleted_post_meta', array( $this, 'metorik_sync_cost' ), 10, 4 );

			// ATUM.
			add_filter( 'alg_wc_cog_copy_product_meta_args', array( $this, 'use_atum_get_cost_function_when_importing' ) );
			add_action( 'atum/data_store/after_save_product_data', array( $this, 'sync_atum_purchase_price_with_cost' ), 10, 2 );
			add_filter( 'alg_wc_cog_comp_atum_cost', array( $this, 'subtract_tax_rate_from_atum_product' ), 10, 2 );

			// Compatibility with Product Add-ons.
			require_once( 'class-alg-wc-cog-pro-product-addons.php' );

			// WPC product bundle.
			add_filter( 'alg_wc_cog_get_product_cost', array( $this, 'calculate_wpc_bundle_cost' ), 10, 2 );
			add_filter( 'alg_wc_cog_order_item_cost', array( $this, 'set_wpc_bundle_order_item_cost_as_zero_on_order_cost_calculation' ), 10, 2 );
			add_action( 'save_post_shop_order', array( $this, 'set_wpc_bundle_order_item_cost_as_zero_on_order_update' ), PHP_INT_MAX, 3 );
			add_action( 'woocommerce_new_order_item', array( $this, 'set_wpc_bundle_order_item_cost_as_zero_on_new_order_item' ), 10, 3 );

			// WooCommerce Measurement Price Calculator.
			add_filter( 'alg_wc_cog_order_item_cost', array( $this, 'wcmpc_adjust_cost_by_measurement' ), 10, 4 );
			add_filter( 'alg_wc_cog_cost_input_label_placeholders', array( $this, 'wcmpc_add_measurement_unit_placeholder' ),10,2 );

			// WooCommerce Food plugin.
			require_once( 'class-alg-wc-cog-pro-compatibility-wc-food.php' );

			// WPSyncSheets plugin.
			require_once( 'class-alg-wc-cog-pro-compatibility-wpsyncsheets.php' );
			$class = new Alg_WC_Cost_of_Goods_Pro_Compatibility_WPSyncSheets();
			$class->init();
		}

		/**
		 * wcmpc_add_measurement_unit_placeholder.
		 *
		 * @version 3.1.9
		 * @since   3.1.9
		 *
		 * @param $placeholders
		 * @param $product_id
		 *
		 * @return mixed
		 */
		function wcmpc_add_measurement_unit_placeholder( $placeholders, $product_id ) {
			if ( 'yes' !== get_option( 'alg_wc_cog_comp_wcmpc_add_measurement_unit_placeholder_to_cost_label', 'no' ) ) {
				return $placeholders;
			}
			if (
				is_a( $product = wc_get_product( $product_id ), 'WC_Product' ) &&
				! empty( $calculator_meta = $product->get_meta( '_wc_price_calculator' ) ) &&
				isset( $calculator_meta['calculator_type'] ) && ! empty( $type = $calculator_meta['calculator_type'] ) &&
				isset( $calculator_meta[ $type ]['pricing']['enabled'] ) && 'yes' === $calculator_meta[ $type ]['pricing']['enabled'] &&
				isset( $calculator_meta[ $type ][ $type ] ) && isset( $calculator_meta[ $type ][ $type ]['unit'] )
			) {
				$placeholders['%measurement_unit%'] = $calculator_meta[ $type ][ $type ]['unit'];
			} elseif ( is_null( $product_id ) ) {
				$placeholders['%measurement_unit%'] = '';
			}

			return $placeholders;
		}

		/**
		 * wcmpc_adjust_cost_by_measurement.
		 *
		 * @version 3.1.9
		 * @since   3.1.9
		 *
		 * @param $cost
		 * @param $product
		 * @param $item_id
		 * @param $item
		 *
		 * @return float
		 */
		function wcmpc_adjust_cost_by_measurement( $cost, $product, $item_id, $item ) {
			if (
				! empty( $measurement_data = $item->get_meta( '_measurement_data' ) ) &&
				'yes' === get_option( 'alg_wc_cog_comp_wcmpc_adjust_cost_by_measure', 'no' ) &&
				isset( $measurement_data['_measurement_needed'] ) &&
				$measurement_needed = $measurement_data['_measurement_needed']
			) {
				$cost *= (float) $measurement_needed;
			}

			return $cost;
		}

		/**
		 * subtract_tax_rate_from_atum_product.
		 *
		 * @version 2.8.7
		 * @since   2.8.7
		 *
		 * @param $atum_cost
		 * @param $product_id
		 *
		 * @return bool|float
		 */
		function subtract_tax_rate_from_atum_product( $atum_cost, $product_id ) {
			if ( 'yes' === get_option( 'alg_wc_cog_comp_atum_subtract_atum_taxes', 'no' ) ) {
				$atum_cost = alg_wc_cog_get_cost_subtracting_tax_rate( array(
					'product_id' => $product_id,
					'cost'       => $atum_cost
				) );
			}
			return $atum_cost;
		}

		/**
		 * sync_atum_purchase_price_with_cost.
		 *
		 * @version 2.8.7
		 * @since   2.8.2
		 *
		 * @param $data
		 * @param $product_id
		 */
		function sync_atum_purchase_price_with_cost( $data, $product_id ) {
			if (
				isset( $data['purchase_price'] ) &&
				! empty( $data['purchase_price'] ) &&
				'yes' === get_option( 'alg_wc_cog_comp_atum_get_sync_purchase_price_with_cost', 'no' )
			) {
				$atum_cost = $data['purchase_price'];
				$atum_cost = apply_filters( 'alg_wc_cog_comp_atum_cost', $atum_cost, $product_id );
				update_post_meta( $product_id, '_alg_wc_cog_cost', $atum_cost );
			}
		}

		/**
		 * use_atum_get_cost_function_when_importing.
		 *
		 * @version 2.8.9
		 * @since   2.5.7
		 *
		 * @param $args
		 *
		 * @return mixed
		 */
		function use_atum_get_cost_function_when_importing( $args ) {
			if (
				'yes' === get_option( 'alg_wc_cog_comp_atum_get_cost_function_enabled', 'no' ) &&
				defined( 'ATUM_VERSION' )
			) {
				$product                   = \Atum\Inc\Helpers::get_atum_product( $args['product_id'] );
				$atum_cost                 = $product->get_purchase_price();
				$atum_cost                 = apply_filters( 'alg_wc_cog_comp_atum_cost', $atum_cost, $args['product_id'] );
				$args['get_cost_function'] = '';
				$args['source_value']      = $atum_cost;
			}
			return $args;
		}

		/**
		 * set_wpc_bundle_order_item_cost_as_zero_on_new_order_item.
		 *
		 * @version 2.6.2
		 * @since   2.6.2
		 *
		 * @param $item_id
		 * @param $item
		 * @param $order_id
		 *
		 * @throws Exception
		 */
		function set_wpc_bundle_order_item_cost_as_zero_on_new_order_item( $item_id, $item, $order_id ) {
			if (
				'yes' === get_option( 'alg_wc_cog_wpcpb_exclude_bundle_cost_from_order_item', 'no' ) &&
				is_a( $item, 'WC_Order_Item_Product' ) &&
				is_a( $item->get_product(), 'WC_Product_Woosb' )
			) {
				wc_update_order_item_meta( $item_id, '_alg_wc_cog_item_cost', 0 );
			}
		}

		/**
		 * set_wpc_bundle_order_item_cost_as_zero_on_order_update.
		 *
		 * @version 3.0.2
		 * @since   2.6.2
		 *
		 * @param $post_id
		 * @param $post
		 * @param $update
		 *
		 * @throws Exception
		 */
		function set_wpc_bundle_order_item_cost_as_zero_on_order_update( $post_id, $post, $update ) {
			if (
				'yes' === get_option( 'alg_wc_cog_wpcpb_exclude_bundle_cost_from_order_item', 'no' ) &&
				is_a( $order = wc_get_order( $post_id ), 'WC_Order' )
			) {
				remove_action( 'save_post_shop_order', array( $this, 'set_wpc_bundle_order_item_cost_as_zero_on_order_update' ), PHP_INT_MAX );
				foreach ( $order->get_items() as $item_id => $item ) {
					if (
						is_a( $item, 'WC_Order_Item_Product' ) &&
						is_a( $item->get_product(), 'WC_Product_Woosb' )
					) {
						wc_update_order_item_meta( $item_id, '_alg_wc_cog_item_cost', 0 );
					}
				}
				add_action( 'save_post_shop_order', array( $this, 'set_wpc_bundle_order_item_cost_as_zero_on_order_update' ), PHP_INT_MAX, 3 );
			}
		}

		/**
		 * set_wpc_bundle_order_item_cost_as_zero_on_order_calculation.
		 *
		 * @version 2.6.2
		 * @since   2.6.1
		 *
		 * @param $cost
		 * @param $product_id
		 *
		 * @return int
		 */
		function set_wpc_bundle_order_item_cost_as_zero_on_order_cost_calculation( $cost, $product_id ) {
			if (
				'yes' === get_option( 'alg_wc_cog_wpcpb_exclude_bundle_cost_from_order_item', 'no' ) &&
				is_a( $product = wc_get_product( $product_id ), 'WC_Product_Woosb' )
			) {
				$cost = 0;
			}
			return $cost;
		}

		/**
		 * get_wpc_bundle_cost.
		 *
		 * @version 2.6.1
		 * @since   2.6.1
		 *
		 * @param $cost
		 * @param $product_id
		 *
		 * @return float
		 */
		function calculate_wpc_bundle_cost( $cost, $product_id ) {
			if (
				'yes' === get_option( 'alg_wc_cog_wpcpb_calculate_cost_from_bundle_items', 'no' ) &&
				is_a( $product = wc_get_product( $product_id ), 'WC_Product_Woosb' ) &&
				! empty( $items = $product->get_items() )
			) {
				$cost = 0;
				foreach ( $items as $item ) {
					$cost += (float) alg_wc_cog()->core->products->get_product_cost( $item['id'] ) * (int) $item['qty'];
				}
			}
			return $cost;
		}

		/**
		 * fix_cost_from_wp_all_import.
		 *
		 * @version 2.9.5
		 * @since   2.3.4
		 *
		 * @param $value
		 * @param $post_id
		 * @param $key
		 * @param $original_value
		 * @param $existing_meta_keys
		 * @param $import_id
		 *
		 * @return mixed
		 */
		function fix_cost_from_wp_all_import( $value, $post_id, $key, $original_value, $existing_meta_keys, $import_id ) {
			if ( '_alg_wc_cog_cost' !== $key ) {
				return $value;
			}
			if ( 'yes' === get_option( 'alg_wc_cog_wp_all_import_convert_to_float', 'no' ) ) {
				$value = alg_wc_cog_sanitize_number( array(
					'number'                    => $value,
					'dots_and_commas_operation' => 'comma-to-dot'
				) );
			}
			if ( 'yes' === get_option( 'alg_wc_cog_wp_all_import_sanitize_float', 'no' ) ) {
				$value = filter_var( $value, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION );
			}
			return $value;
		}

		/**
		 * exclude_wpc_bundle_from_stock_report.
		 *
		 * @version 2.3.8
		 * @since   2.3.8
		 *
		 * @param $args
		 *
		 * @return mixed
		 */
		function exclude_wpc_bundle_from_stock_report( $args ) {
			if ( 'yes' === get_option( 'alg_wc_cog_wpc_product_bundle_for_wc', 'no' ) ) {
				$tax_query         = isset( $args['tax_query'] ) ? $args['tax_query'] : array();
				$tax_query[]       = array(
					'taxonomy' => 'product_type',
					'field'    => 'slug',
					'terms'    => 'woosb',
					'operator' => 'NOT IN'
				);
				$args['tax_query'] = $tax_query;
			}
			return $args;
		}

		/**
		 * exclude_wpc_bundle_from_orders_report.
		 *
		 * @version 2.3.8
		 * @since   2.3.8
		 *
		 * @param $args
		 *
		 * @return mixed
		 */
		function exclude_wpc_bundle_from_orders_report( $args ) {
			if (
				'yes' === get_option( 'alg_wc_cog_wpc_product_bundle_for_wc', 'no' )
				&& isset( $_GET['page'] )
				&& 'wc-reports' == $_GET['page']
				&& isset( $_GET['tab'] )
				&& 'orders' == $_GET['tab']
				&& isset( $_GET['report'] )
				&& 'alg_cost_of_goods' == $_GET['report']
				&& isset( $args['join'] )
				&& false !== strpos( strtolower( $args['join'] ), strtolower( 'wp_woocommerce_order_itemmeta AS order_item_meta__qty' ) )
				&& isset( $args['select'] )
				&& false !== strpos( strtolower( $args['select'] ), strtolower( 'SUM( order_item_meta__qty.meta_value)' ) )
			) {
				global $wpdb;
				$where = "AND order_item_meta__qty.order_item_id NOT IN(
				SELECT DISTINCT wcoim.order_item_id
				FROM {$wpdb->prefix}woocommerce_order_itemmeta AS wcoim
				WHERE wcoim.meta_key = '_woosb_ids'
			)";
				if ( isset( $args['where'] ) ) {
					$args['where'] .= " {$where}";
				} else {
					$args['where'] = "{$where}";
				}
			}
			return $args;
		}

		/**
		 * Setup orders reports adding compatibility with Openpos - WooCommerce Point Of Sale(POS).
		 *
		 * @version 3.2.4
		 * @since   2.3.9
		 *
		 * @param $query
		 *
		 * @return mixed
		 */
		function setup_openpos_order_reports_query( $query ) {
			global $wpdb;
			if (
				! empty( $order_types = get_option( 'alg_wc_cog_openpos_anhvnit_report_order_type', array() ) ) &&
				isset( $_GET['report'] ) &&
				'alg_cost_of_goods' === $_GET['report']
			) {
				if ( in_array( 'common_orders', $order_types ) && ! in_array( 'openpos_orders', $order_types ) ) {
					$query['join']  .= " LEFT JOIN {$wpdb->postmeta} AS alg_wc_cog__op_order_source
				ON ( posts.ID = alg_wc_cog__op_order_source.post_id
				AND alg_wc_cog__op_order_source.meta_key = '_op_order_source' )";
					$query['where'] .= " AND alg_wc_cog__op_order_source.meta_value IS NULL OR( alg_wc_cog__op_order_source.post_id IS NOT NULL && alg_wc_cog__op_order_source.meta_value != 'openpos')";
				} elseif ( ! in_array( 'common_orders', $order_types ) && in_array( 'openpos_orders', $order_types ) ) {
					$query['join'] .= " INNER JOIN {$wpdb->postmeta} AS alg_wc_cog__op_order_source
				ON ( posts.ID = alg_wc_cog__op_order_source.post_id
				AND alg_wc_cog__op_order_source.meta_key = '_op_order_source' )";
				}
			}

			return $query;
		}

		/**
		 * Syncs `_alg_wc_cog_cost` with `_wc_cog_cost`.
		 *
		 * @version 2.8.2
		 * @since   2.4.7
		 *
		 * @param $meta_id
		 * @param $post_id
		 * @param $meta_key
		 * @param $meta_value
		 */
		function metorik_sync_cost( $meta_id, $post_id, $meta_key, $meta_value ) {
			if (
				'_alg_wc_cog_cost' === $meta_key &&
				'yes' === get_option( 'alg_wc_cog_metorik_sync_cost_with_wc_cog_cost', 'no' )
			) {
				if ( 'deleted_post_meta' === current_action() ) {
					delete_post_meta( $post_id, '_wc_cog_cost' );
				} elseif ( 'added_post_meta' === current_action() || 'updated_post_meta' === current_action() ) {
					update_post_meta( $post_id, '_wc_cog_cost', $meta_value );
				}
			}
		}
	}
}
return new Alg_WC_Cost_of_Goods_Pro_Compatibility();