<?php
/**
 * Cost of Goods for WooCommerce - Extra currencies costs.
 *
 * @see https://exthemes.net/woocommerce-food/
 *
 * @version 3.0.2
 * @since   2.8.7
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro_Extra_Currencies_Costs' ) ) {

	class Alg_WC_Cost_of_Goods_Pro_Extra_Currencies_Costs {

		/**
		 * is_currencies_costs_enabled.
		 *
		 * @since 2.8.7
		 *
		 * @var null
		 */
		protected $is_currencies_costs_enabled = null;

		/**
		 * Initializes the class.
		 *
		 * @version 2.8.7
		 * @since   2.8.7
		 */
		function init() {
			// Adjust order values.
			add_filter( 'alg_wc_cog_update_order_values', array( $this, 'adjust_order_values' ), 10, 2 );
			// Order metabox.
			add_filter( 'alg_wc_cog_cost_meta_keys', array( $this, 'add_order_meta_box_meta_key' ) );
		}

		/**
		 * add_order_meta_box_meta_key.
		 *
		 * @version 2.8.7
		 * @since   2.8.7
		 *
		 * @param $keys
		 *
		 * @return mixed
		 */
		function add_order_meta_box_meta_key( $keys ) {
			if ( $this->is_currencies_costs_enabled() ) {
				$keys['_alg_wc_cog_currencies_costs_total'] = __( 'Currencies costs', 'cost-of-goods-for-woocommerce' );
			}
			return $keys;
		}

		/**
		 * adjust_order_values.
		 *
		 * @version 3.0.2
		 * @since   2.8.7
		 *
		 * @param $order_values
		 * @param $info
		 *
		 * @return mixed
		 */
		function adjust_order_values( $order_values, $info ) {
			if ( $this->is_currencies_costs_enabled() ) {
				$order_id               = $info['order_id'];
				$order                  = $info['order'];
				$order_currency         = $order->get_meta( '_order_currency', true );
				$fixed_cost             = ! empty( $fixed_costs_currencies = $this->get_currencies_with_fixed_costs() ) ? $fixed_costs_currencies[ $order_currency ] : 0;
				$percent_cost           = ! empty( $percent_costs_currencies = $this->get_currencies_with_percent_costs() ) ? $percent_costs_currencies[ $order_currency ] : 0;
				$percent_cost           = apply_filters( 'alg_wc_cog_currencies_costs_percentage_total', $order->get_total() * ( $percent_cost / 100 ), $order );
				$currencies_costs_total = (float) apply_filters( 'alg_wc_cog_currencies_costs_total', (float) $percent_cost + (float) $fixed_cost, $order_id );
				$order->update_meta_data( '_alg_wc_cog_currencies_costs_total', (float) $currencies_costs_total );
				$order->save();
				$order_values['total_cost'] += $currencies_costs_total;
				$order_values['profit']     -= $currencies_costs_total;
			}
			return $order_values;
		}

		/**
		 * get_currencies_with_fixed_costs.
		 *
		 * @version 2.8.7
		 * @since   2.8.7
		 *
		 * @return array
		 */
		function get_currencies_with_fixed_costs() {
			return array_filter( get_option( 'alg_wc_cog_currencies_costs_fixed', array() ) );
		}

		/**
		 * get_currencies_with_percent_costs.
		 *
		 * @version 2.8.7
		 * @since   2.8.7
		 *
		 * @return array
		 */
		function get_currencies_with_percent_costs() {
			return array_filter( get_option( 'alg_wc_cog_currencies_costs_percent', array() ) );
		}

		/**
		 * is_add_stock_enabled.
		 *
		 * @version 2.8.7
		 * @since   2.8.7
		 *
		 * @return bool|null
		 */
		function is_currencies_costs_enabled() {
			if ( is_null( $this->is_currencies_costs_enabled ) ) {
				$this->is_currencies_costs_enabled = ( 'yes' === get_option( 'alg_wc_cog_currencies_costs_enabled', 'no' ) );
			}
			return $this->is_currencies_costs_enabled;
		}
	}
}

return new Alg_WC_Cost_of_Goods_Pro_Extra_Currencies_Costs();