<?php
/**
 * Cost of Goods for WooCommerce - Product Addons compatibility.
 *
 * @version 3.0.2
 * @since   2.4.3
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro_Product_Addons' ) ) :

	class Alg_WC_Cost_of_Goods_Pro_Product_Addons {

		/**
		 * Constructor.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 * @todo    [next] better "Analytics" support
		 * @todo    [next] add more information to our "Import tool"
		 */
		function __construct() {
			add_action( 'woocommerce_product_addons_panel_option_row', array( $this, 'add_cost_field' ), 10, 4 );
			add_action( 'woocommerce_product_addons_panel_option_heading', array( $this, 'add_heading' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'script_styles' ), 100 );
			add_filter( 'woocommerce_product_addons_save_data', array( $this, 'save_data' ), 10, 2 );
			add_action( 'alg_wc_cog_before_update_order_items_costs', array( $this, 'update_prod_addons_costs_order_meta' ) );
			add_filter( 'woocommerce_hidden_order_itemmeta', array( $this, 'hidden_order_itemmeta' ), 50 );
		}

		/**
		 * can_use_addon_option.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 *
		 * @return bool
		 */
		function can_use_addon_option() {
			if (
				'yes' === get_option( 'alg_wc_cog_product_addons', 'no' )
				&& class_exists( '\WC_Product_Addons_Helper' )
			) {
				return true;
			}
			return false;
		}

		/**
		 * hidden_order_itemmeta.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 *
		 * @param $args
		 *
		 * @return array
		 */
		function hidden_order_itemmeta( $args ) {
			$args[] = '_alg_wc_cog_pao_costs';
			return $args;
		}

		/**
		 * update product addons costs meta in order.
		 *
		 * @version 3.0.2
		 * @since   2.4.3
		 *
		 * @param $order
		 *
		 * @throws Exception
		 */
		function update_prod_addons_costs_order_meta( $order ) {
			if ( ! $this->can_use_addon_option() ) {
				return;
			}
			$total_costs = 0;
			foreach ( $order->get_items() as $item_id => $item ) {
				$addons           = \WC_Product_Addons_Helper::get_product_addons( $item->get_product_id() );
				$meta_data        = $item->get_meta_data();
				$meta_data_values = wp_list_pluck( $meta_data, 'value' );
				if ( is_array( $addons ) && count( $addons ) > 0 ) {
					$item_costs = 0;
					foreach ( $addons as $addon ) {
						foreach ( $addon['options'] as $option ) {
							$label = $option['label'];
							if ( in_array( $label, $meta_data_values ) ) {
								$total_costs += (float) $option['_alg_wc_cog_cost'];
								$item_costs  += (float) $option['_alg_wc_cog_cost'];
							}
						}
					}
					wc_update_order_item_meta( $item_id, '_alg_wc_cog_pao_costs', $item_costs );
				}
			}
			$order->calculate_totals(); // Recalculate Order totals and save
			if ( $total_costs > 0 ) {
				$order->update_meta_data( '_alg_wc_cog_pao_costs', $total_costs );
				$order->save();
			}
		}

		/**
		 * save_data.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 *
		 * @param $data
		 * @param $i
		 *
		 * @return mixed
		 */
		function save_data( $data, $i ) {
			if ( ! $this->can_use_addon_option() ) {
				return $data;
			}
			$addon_cost = isset( $_POST['product_addon_option_alg_wc_cog_cost'] ) ? $_POST['product_addon_option_alg_wc_cog_cost'] : array();
			foreach ( $data['options'] as $key => $opt ) {
				if ( isset( $addon_cost[ $i ][ $key ] ) ) {
					$data['options'][ $key ]['_alg_wc_cog_cost'] = wc_format_decimal( sanitize_text_field( stripslashes( $addon_cost[ $i ][ $key ] ) ) );
				}
			}
			return $data;
		}

		/**
		 * script_styles.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 */
		function script_styles(){
			if ( ! $this->can_use_addon_option() ) {
				return;
			}
			if (
				'product_page_addons' !== get_current_screen()->id &&
				'product'             !== get_current_screen()->id &&
				'shop_order'          !== get_current_screen()->id &&
				'shop_subscription'   !== get_current_screen()->id
			) {
				return;
			}
			?>
			<style>
				.wc-pao-addon-content-price{
					grid-column: 11/12 !important;
				}
				.alg-wc-cog-content-cost{
					grid-column:12/13 !important;
				}
				.wc-pao-addon-content-price-header{
					grid-column:8/12 !important;
				}
				.alg-wc-cog-content-cost-header{
					grid-column:12/14 !important;
				}
			</style>
			<?php
		}

		/**
		 * add_heading.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 */
		function add_heading(){
			if ( ! $this->can_use_addon_option() ) {
				return;
			}
			?>
			<div class="wc-pao-addon-content-price-header alg-wc-cog-content-cost-header">
				<div class="wc-pao-addon-content-price-wrap alg-wc-cog-content-cost-wrap">
					<?php esc_html_e( 'Cost', 'woocommerce-product-addons' ); ?>
					<?php //echo wc_help_tip( __( 'Choose how to calculate price: apply a flat fee regardless of quantity, charge per quantity ordered, or charge a percentage of the total', 'woocommerce-product-addons' ) ); ?>
				</div>
			</div>
			<?php
		}

		/**
		 * add_cost_field.
		 *
		 * @version 2.4.3
		 * @since   2.4.3
		 *
		 * @param $post
		 * @param $product_addons
		 * @param $loop
		 * @param $option
		 */
		function add_cost_field( $post, $product_addons, $loop, $option ) {
			if ( ! $this->can_use_addon_option() ) {
				return;
			}
			$opt_cost              = ! empty( $option['_alg_wc_cog_cost'] ) ? $option['_alg_wc_cog_cost'] : '';
			$opt_decimal_separator = wc_get_price_decimal_separator();
			?>
			<div class="wc-pao-addon-content-price alg-wc-cog-content-cost">
				<input type="text" name="product_addon_option_alg_wc_cog_cost[<?php echo $loop; ?>][]" value="<?php echo esc_attr( wc_format_localized_price( $opt_cost ) ); ?>" placeholder="0<?php echo esc_attr( $opt_decimal_separator ); ?>00" class="wc_input_price"/>
			</div>
			<?php
		}
	}
endif;
return new Alg_WC_Cost_of_Goods_Pro_Product_Addons();