<?php
/**
 * Cost of Goods for WooCommerce - Pro - Quick and bulk edit.
 *
 * @version 3.2.6
 * @since   3.2.3
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro_Quick_And_Bulk_Edit' ) ) {

	class Alg_WC_Cost_of_Goods_Pro_Quick_And_Bulk_Edit {

		/**
		 * $do_add_cost_quick_edit.
		 *
		 * @since 2.9.4
		 */
		public $do_add_cost_quick_edit;

		/**
		 * $do_add_cost_bulk_edit.
		 *
		 * @since 2.9.4
		 */
		public $do_add_cost_bulk_edit;

		/**
		 * Admin quick bulk edit position.
		 *
		 * @since 2.9.4
		 */
		public $cost_quick_bulk_edit_position;

		/**
		 * Initializes the class.
		 *
		 * @version 3.2.3
		 * @since   3.2.3
		 *
		 * @return void
		 */
		function init() {
			// Add and save quick and bulk edit fields for Cost.
			$this->cost_quick_bulk_edit_position = 'end';
			add_action( 'woocommerce_product_quick_edit_' . $this->cost_quick_bulk_edit_position, array( $this, 'add_cost_bulk_and_quick_edit_fields' ), PHP_INT_MAX );
			add_action( 'woocommerce_product_bulk_edit_' . $this->cost_quick_bulk_edit_position, array( $this, 'add_cost_bulk_and_quick_edit_fields' ), PHP_INT_MAX );
			add_action( 'woocommerce_product_bulk_and_quick_edit', array( $this, 'save_cost_bulk_and_quick_edit_fields' ), PHP_INT_MAX, 2 );

			// Populate cost quick fields.
			add_action( 'wp_ajax_alg_wc_cog_get_quick_edit_field_value', array( $this, 'get_field_value_from_quick_edit' ) );
			add_action( 'admin_footer', array( $this, 'populate_cost_quick_edit_via_js' ) );

			// "Add stock" fields in quick and bulk edit.
			add_action( 'woocommerce_product_bulk_edit_' . 'end', array( $this, 'add_addstock_bulk_and_quick_edit_fields' ), PHP_INT_MAX );
			add_action( 'woocommerce_product_quick_edit_' . 'end', array( $this, 'add_addstock_bulk_and_quick_edit_fields' ), PHP_INT_MAX );
			add_action( 'woocommerce_product_bulk_and_quick_edit', array( $this, 'save_addstock_bulk_and_quick_edit_fields' ), PHP_INT_MAX, 2 );
		}

		/**
		 * add_addstock_bulk_and_quick_edit_fields.
		 *
		 * @version 2.6.4
		 * @since   2.3.5
		 */
		function add_addstock_bulk_and_quick_edit_fields() {
			if (
				(
					false !== strpos( current_filter(), 'woocommerce_product_bulk_edit' )
					&& 'yes' == get_option( 'alg_wc_cog_products_add_stock_bulk_edit', 'no' )
				)
				||
				( false !== strpos( current_filter(), 'woocommerce_product_quick_edit' )
				  && 'yes' == get_option( 'alg_wc_cog_products_add_stock_quick_edit', 'no' )
				)
			) {
				$html = $this->get_add_stock_bulk_and_quick_edit_fields();
				echo $html;
			}
		}

		/**
		 * get_add_stock_bulk_and_quick_edit_fields.
		 *
		 * @version 2.4.2
		 * @since   2.3.5
		 *
		 * @todo Try to understand why the tip doesn't work on quick edit but does work on bulk edit.
		 *
		 * @return string
		 */
		function get_add_stock_bulk_and_quick_edit_fields() {
			$negative_stock_allowed = 'yes' === get_option( 'alg_wc_cog_products_add_stock_negative_stock', 'no' );
			$add_stock_input_min = $negative_stock_allowed ? '' : 'min="0"';
			//$tip = wc_help_tip( __( '"Stock" will be added to your inventory, and "Cost" will be used to calculate new average cost of goods for the product.', 'cost-of-goods-for-woocommerce' ) );
			ob_start();
			?>
            <br class="clear"/>
            <h4 class="title" style="margin-bottom: 10px;"><?php echo __( 'Cost of Goods', 'cost-of-goods-for-woocommerce' ) . ': ' . __( 'Add stock', 'cost-of-goods-for-woocommerce' ) ?><?php //echo $tip; ?></h4>
            <label>
                <span class="title"><?php echo __( 'Stock', 'cost-of-goods-for-woocommerce' ) ?></span>
                <span class="input-text-wrap">
				<input name="_alg_wc_cog_add_stock_qb" id="_alg_wc_cog_add_stock_qb" class="short" type="number" <?php echo $add_stock_input_min; ?>>
			</span>
            </label>
            <label>
                <span class="title"><?php echo __( 'Cost', 'cost-of-goods-for-woocommerce' ) ?></span>
                <span class="input-text-wrap">
				<input name="_alg_wc_cog_add_stock_cost_qb" id="_alg_wc_cog_add_stock_cost_qb" class="short wc_input_price" type="number" step="0.0001" min="0">
			</span>
            </label>
			<?php
			return ob_get_clean();
		}

		/**
		 * save_addstock_bulk_and_quick_edit_fields.
		 *
		 * @version 3.1.7
		 * @since   2.3.5
		 */
		function save_addstock_bulk_and_quick_edit_fields( $post_id, $post ) {
			// If this is an autosave, our form has not been submitted, so we don't want to do anything.
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return $post_id;
			}
			// Don't save revisions and autosaves.
			if ( wp_is_post_revision( $post_id ) || wp_is_post_autosave( $post_id ) || 'product' !== $post->post_type || ! current_user_can( 'edit_post', $post_id ) ) {
				return $post_id;
			}
			// Check nonce.
			if ( ! isset( $_REQUEST['woocommerce_quick_edit_nonce'] ) || ! wp_verify_nonce( $_REQUEST['woocommerce_quick_edit_nonce'], 'woocommerce_quick_edit_nonce' ) ) { // WPCS: input var ok, sanitization ok.
				return $post_id;
			}
			// Check bulk or quick edit.
			if ( ! empty( $_REQUEST['woocommerce_quick_edit'] ) && 'no' == get_option( 'alg_wc_cog_products_add_stock_quick_edit', 'no' ) ) {
				return $post_id;
			} else {
				if ( 'no' == get_option( 'alg_wc_cog_products_add_stock_bulk_edit', 'no' ) ) {
					return $post_id;
				}
			}
			// Save
			if (
				isset( $_REQUEST['_alg_wc_cog_add_stock_qb'] ) && ! empty( $stock = $_REQUEST['_alg_wc_cog_add_stock_qb'] )
				&&
				(
					'do_nothing' !== ( $empty_cost_action = get_option( 'alg_wc_cog_products_add_stock_empty_cost_action', 'do_nothing' ) )
					||
					(
						'do_nothing' === $empty_cost_action
						&& isset( $_REQUEST['_alg_wc_cog_add_stock_cost_qb'] ) && ! empty( $_REQUEST['_alg_wc_cog_add_stock_cost_qb'] )
					)
				)
				//&& isset( $_REQUEST['_alg_wc_cog_add_stock_cost_qb'] ) && ! empty( $cost = $_REQUEST['_alg_wc_cog_add_stock_cost_qb'] )
			) {
				$cost = isset( $_REQUEST['_alg_wc_cog_add_stock_cost_qb'] ) ? $_REQUEST['_alg_wc_cog_add_stock_cost_qb'] : '';
				alg_wc_cog()->core->products_add_stock->product_add_stock( array(
					'product_id' => (int) $post_id,
					'stock'      => wc_clean( wp_unslash( $stock ) ),
					'cost'       => wc_clean( wp_unslash( $cost ) ),
				) );
			}
			return $post_id;
		}

		/**
		 * add_cost_bulk_and_quick_edit_fields.
		 *
		 * @version 1.7.0
		 * @since   1.7.0
		 * @todo    [maybe] reposition this (to the `price_fields` section)
		 * @todo    [maybe] actual value (instead of "No change" placeholder) (probably need to add value to `woocommerce_inline_`)
		 */
		function add_cost_bulk_and_quick_edit_fields() {
			if (
				( $this->need_to_add_cost_to_quick_edit() && strpos( current_filter(), 'quick_edit' ) !== false ) ||
				( $this->need_to_add_cost_to_bulk_edit() && strpos( current_filter(), 'bulk_edit' ) !== false )
			) {
				$current_filter = current_filter();
				if ( 'end' === $this->cost_quick_bulk_edit_position && 'woocommerce_product_quick_edit_end' === $current_filter ) {
					echo '<br class="clear" />';
				}
				?><label>
                <span class="title"><?php echo __( 'Cost', 'cost-of-goods-for-woocommerce' ); ?></span>
                <span class="input-text-wrap">
				<input type="text" name="_alg_wc_cog_cost_qb" class="text wc_input_price"
                       placeholder="<?php echo __( '- No change -', 'cost-of-goods-for-woocommerce' ); ?>" value="">
			    </span>
                </label><?php
				if ( 'start' === $this->cost_quick_bulk_edit_position && 'woocommerce_product_quick_edit_start' === $current_filter ) {
					echo '<br class="clear" />';
				}
			}
		}

		/**
		 * save_cost_bulk_and_quick_edit_fields.
		 *
		 * @version 3.0.7
		 * @since   1.7.0
		 */
		function save_cost_bulk_and_quick_edit_fields( $post_id, $post ) {
			if (
				$this->need_to_add_cost_to_quick_edit() ||
				$this->need_to_add_cost_to_bulk_edit()
			) {

				// If this is an autosave, our form has not been submitted, so we don't want to do anything.
				if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
					return $post_id;
				}
				// Don't save revisions and autosaves.
				if ( wp_is_post_revision( $post_id ) || wp_is_post_autosave( $post_id ) || 'product' !== $post->post_type || ! current_user_can( 'edit_post', $post_id ) ) {
					return $post_id;
				}
				// Check nonce.
				if ( ! isset( $_REQUEST['woocommerce_quick_edit_nonce'] ) || ! wp_verify_nonce( $_REQUEST['woocommerce_quick_edit_nonce'], 'woocommerce_quick_edit_nonce' ) ) { // WPCS: input var ok, sanitization ok.
					return $post_id;
				}
				// Check bulk or quick edit.
				if ( ! empty( $_REQUEST['woocommerce_quick_edit'] ) ) { // WPCS: input var ok.
					if ( ! $this->do_add_cost_quick_edit ) {
						return $post_id;
					}
				} else {
					if ( ! $this->do_add_cost_bulk_edit ) {
						return $post_id;
					}
				}
				// Save.
				if ( isset( $_REQUEST['_alg_wc_cog_cost_qb'] ) && '' !== $_REQUEST['_alg_wc_cog_cost_qb'] ) {
					update_post_meta( $post_id, '_alg_wc_cog_cost', wc_clean( $_REQUEST['_alg_wc_cog_cost_qb'] ) );
					// Update all variations
					if (
						'yes' === get_option( 'alg_wc_cog_products_quick_edit_replace_variations', 'no' )
						&& ! empty( $product = wc_get_product( $post_id ) )
						&& $product->is_type( 'variable' )
					) {
						foreach ( $product->get_children() as $variation_id ) {
							update_post_meta( $variation_id, '_alg_wc_cog_cost', wc_clean( $_REQUEST['_alg_wc_cog_cost_qb'] ) );
						}
					}
				}
			}

			return $post_id;
		}

		/**
		 * get_field_value_from_quick_edit.
		 *
		 * @version 3.2.3
		 * @since   3.2.3
		 *
		 * @return void
		 */
		function get_field_value_from_quick_edit() {
			check_ajax_referer( 'alg-wc-cog-quick-edit-nonce', 'security' );

			$args = wp_parse_args( $_POST, array(
				'field_type' => 'product_cost',
				'post_id'    => ''
			) );

			$field_type = $args['field_type'];
			$post_id    = intval( $args['post_id'] );
			if (
				'product_cost' === $field_type &&
				! empty( $post_id ) &&
				is_a( wc_get_product( $post_id ), 'WC_Product' )
			) {
				wp_send_json_success( array(
					'value' => alg_wc_cog()->core->products->get_product_cost( $post_id )
				) );
			}
			wp_send_json_error();
		}

		/**
		 * populate_cost_quick_edit_via_js.
		 *
		 * @version 3.2.6
		 * @since   3.2.3
		 *
		 * @return void
		 * @link    https://rudrastyh.com/wordpress/quick-edit-tutorial.html#populate-columns
		 *
		 */
		function populate_cost_quick_edit_via_js() {
			global $pagenow;
			if (
				! is_admin() ||
				'edit.php' !== $pagenow ||
				! isset( $_GET['post_type'] ) ||
				'product' !== $_GET['post_type'] ||
				! $this->need_to_add_cost_to_quick_edit()
			) {
				return;
			}
			$php_to_js = array(
				'security' => wp_create_nonce( 'alg-wc-cog-quick-edit-nonce' ),
				'action'   => 'alg_wc_cog_get_quick_edit_field_value'
			);
			?>
            <script>
                jQuery(document).ready(function ($) {
                    const wp_inline_edit_function = inlineEditPost.edit;

                    inlineEditPost.edit = function (post_id) {
                        let dataFromPHP = <?php echo json_encode( $php_to_js );?>;

                        // let's merge arguments of the original function
                        wp_inline_edit_function.apply(this, arguments);

                        // get the post ID from the argument
                        if (typeof (post_id) == 'object') { // if it is object, get the ID number
                            post_id = parseInt(this.getId(post_id));
                        }

                        let data = {
                            action: dataFromPHP.action,
                            security: dataFromPHP.security,
                            field_type: 'product_cost',
                            post_id: post_id
                        };

                        let costInput = $('#edit-' + post_id + ' input[name="_alg_wc_cog_cost_qb"]');

                        if (costInput.length && '' == costInput.val()) {
                            $.post(ajaxurl, data, function (response) {
                                if(response.data.value){
                                    costInput.val(response.data.value)
                                }
                            });
                        }
                    }
                });
            </script>
			<?php
		}

		/**
		 * need_to_add_cost_to_quick_edit.
		 *
		 * @version 3.2.3
		 * @since   3.2.3
		 *
		 * @return bool
		 */
		function need_to_add_cost_to_quick_edit() {
			if ( is_null( $this->do_add_cost_quick_edit ) ) {
				$this->do_add_cost_quick_edit = ( 'yes' === get_option( 'alg_wc_cog_products_quick_edit', 'no' ) );
			}

			return $this->do_add_cost_quick_edit;
		}

		/**
		 * need_to_add_cost_to_bulk_edit.
		 *
		 * @version 3.2.3
		 * @since   3.2.3
		 *
		 * @return bool
		 */
		function need_to_add_cost_to_bulk_edit() {
			if ( is_null( $this->do_add_cost_quick_edit ) ) {
				$this->do_add_cost_bulk_edit = ( 'yes' === get_option( 'alg_wc_cog_products_bulk_edit', 'no' ) );
			}

			return $this->do_add_cost_bulk_edit;
		}
	}
}

return new Alg_WC_Cost_of_Goods_Pro_Quick_And_Bulk_Edit();