<?php
/**
 * Cost of Goods for WooCommerce - Pro Class.
 *
 * @version 3.1.7
 * @since   1.4.0
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro' ) ) :

	class Alg_WC_Cost_of_Goods_Pro {

		/**
		 * @var Alg_WC_Cost_of_Goods_Recalculate_Orders_Bkg_Process
		 */
		protected $recalculate_orders_bkg_process;

		/**
		 * Core.
		 *
		 * @since 2.9.4
		 *
		 * @var Alg_WC_Cost_of_Goods_Core
		 */
		public $core;

		/**
		 * Constructor.
		 *
		 * @version 2.8.4
		 * @since   1.4.0
		 * @todo    [next] better "Analytics" support
		 * @todo    [next] add more information to our "Import tool"
		 */
		function __construct() {
			add_filter( 'alg_wc_cog_settings',                           array( $this, 'settings' ), 10, 3 );
			add_filter( 'alg_wc_cog_recalculate_orders_cost_and_profit', array( $this, 'recalculate_orders_cost_and_profit' ) );
			add_filter( 'alg_wc_cog_core_loaded',                        array( $this, 'core_loaded' ) );
			add_action( 'admin_init',                                    array( $this, 'admin_actions' ) );

			// Sanitize restrict by user role option.
			add_filter( 'woocommerce_admin_settings_sanitize_option_' . 'alg_wc_cog_allowed_user_roles', array( $this, 'append_administrator_to_restrict_by_user_role_option' ), 10, 3 );

			// Do not show COG data for disallowed users.
			add_filter( 'alg_wc_cog_create_import_tool_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_edit_costs_tool_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_orders_columns_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_product_columns_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_wc_settings_tab_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_report_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_analytics_orders_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_order_meta_box_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_create_product_meta_box_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_add_cost_input_validation', array( $this, 'block_disallowed_users' ) );
			add_filter( 'alg_wc_cog_add_handling_fee_input_validation', array( $this, 'block_disallowed_users' ) );

			// Fix cost meta query for stock report.
			add_filter( 'alg_wc_cog_stock_report_args', array( $this, 'stock_report_meta_query' ) );

			// Save "Multicurrency" rate from exchangerate-api.com.
			add_action( 'admin_init', array( $this, 'currency_rate_cron_activation' ) );
			add_action( 'alg_wc_cog_currency_rate_update', array( $this, 'save_multicurrency_rate' ) );

			// Compatibility.
			$compatibility = require_once( 'class-alg-wc-cog-pro-compatibility.php' );
			$compatibility->init();

			// Currencies extra costs.
			$currencies_extra_costs = require_once( 'class-alg-wc-cog-pro-extra-currencies-costs.php' );
			$currencies_extra_costs->init();

			// Quick and bulk edit.
			$quick_and_bulk_edit = require_once( 'class-alg-wc-cog-pro-quick-and-bulk-edit.php' );
			$quick_and_bulk_edit->init();
		}

		/**
		 * Changes '_price' and '_alg_wc_cog_cost' meta keys 'type' from 'DECUNAL to 'CHAR'
		 *
		 * @version 2.4.0
		 * @since   2.4.0
		 *
		 * @param $args
		 *
		 * @return mixed
		 */
		function stock_report_meta_query( $args ) {
			if (
				'currency_as_char_and_not_empty' == get_option( 'alg_wc_cog_report_stock_meta_query', 'default' )
				&& isset( $args['meta_query'] )
				&& ! empty( $meta_query = $args['meta_query'] )
				&& ! empty( $decimal_params = wp_list_filter( $meta_query, array( 'type' => 'DECIMAL' ) ) )
			) {
				foreach ( $decimal_params as $key => $query ) {
					if ( in_array( $query['key'], array( '_price', '_alg_wc_cog_cost' ) ) ) {
						$args['meta_query'][ $key ]['compare'] = 'NOT IN';
						$args['meta_query'][ $key ]['value']   = array( '', '0' );
						$args['meta_query'][ $key ]['type']    = 'CHAR';
					}
				}
			}
			return $args;
		}

		/**
		 * append_administrator_to_restrict_by_user_role_option.
		 *
		 * @version 2.3.4
		 * @since   2.3.4
		 *
		 * @param $value
		 * @param $option
		 * @param $raw_value
		 *
		 * @return array
		 */
		function append_administrator_to_restrict_by_user_role_option( $value, $option, $raw_value ) {
			if ( ! empty( $value ) && ! in_array( 'administrator', $value ) ) {
				$value[] = 'administrator';
			}
			return $value;
		}

		/**
		 * block_disallowed_users.
		 *
		 * @version 2.3.4
		 * @since   2.3.4
		 *
		 * @param $validation
		 *
		 * @return bool
		 */
		function block_disallowed_users( $validation ) {
			if ( alg_wc_cog_is_user_allowed() ) {
				return $validation;
			}
			$validation = false;
			return $validation;
		}

		/**
		 * init_bkg_process.
		 *
		 * @version 2.3.0
		 * @since   2.3.0
		 */
		function init_bkg_process() {
			require_once( alg_wc_cog()->plugin_path() . '/includes/background-process/class-alg-wc-cog-recalculate-orders-bkg-process.php' );
			$this->recalculate_orders_bkg_process = new Alg_WC_Cost_of_Goods_Recalculate_Orders_Bkg_Process();
		}

		/**
		 * strip_tags_walker.
		 *
		 * @version 1.5.2
		 * @since   1.5.2
		 */
		function strip_tags_walker( &$item, $key ) {
			$item = strip_tags( $item );
		}

		/**
		 * admin_actions.
		 *
		 * @version 2.1.0
		 * @since   1.5.2
		 * @todo    [next] nonce
		 * @todo    [next] export: replace double quotes
		 * @todo    [next] check if `$data` is not empty
		 */
		function admin_actions() {
			if ( ! empty( $_GET['alg_wc_cog_stock_report_action'] ) && ! empty( $_GET['alg_wc_cog_stock_report_action_param'] ) && current_user_can( 'manage_woocommerce' ) ) {
				$action = sanitize_key( $_GET['alg_wc_cog_stock_report_action'] );
				$param  = sanitize_key( $_GET['alg_wc_cog_stock_report_action_param'] );
				if ( in_array( $param, array( 'totals', 'products' ) ) ) {
					require_once( 'reports/class-wc-report-alg-cog-stock.php' );
					$report = new WC_Report_Alg_Cost_Of_Goods_Stock();
					$data   = $report->output_report( $param, ( 'print' === $action ) );
					array_walk_recursive( $data, array( $this, 'strip_tags_walker' ) );
					switch ( $action ) {
						case 'print':
							echo '<pre>' . alg_wc_cog_get_table_html( $data, array( 'table_heading_type' => 'none' ) ) . '</pre>';
							die();
						case 'export':
							$csv = '';
							foreach ( $data as $record ) {
								$csv .= '"' . implode( '","', $record ) . '"' . PHP_EOL;
							}
							header( 'Content-Description: File Transfer' );
							header( 'Content-Type: application/octet-stream' );
							header( 'Content-Disposition: attachment; filename=cost-of-goods-stock-report-' . $param . '.csv' );
							header( 'Content-Transfer-Encoding: binary' );
							header( 'Expires: 0' );
							header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
							header( 'Pragma: public' );
							header( 'Content-Length: ' . strlen( $csv ) );
							echo $csv;
							die();
					}
				}
			}
		}

		/**
		 * core_loaded.
		 *
		 * @version 2.9.1
		 * @since   1.4.0
		 * @todo    [next] `wmc_admin_reports`
		 * @todo    [later] `$this->admin_quick_bulk_edit_position`: optional 'start'
		 */
		function core_loaded( $core ) {
			$this->core = $core;
			// Recalculate orders cost and profit
			add_filter( 'alg_wc_cog_save_settings',  array( $this, 'recalculate_orders_cost_and_profit' ), PHP_INT_MAX );
			// Reports
			add_filter( 'woocommerce_admin_reports', array( $this, 'add_cog_reports' ), PHP_INT_MAX );

			// Multicurrency
			if ( 'yes' === get_option( 'alg_wc_cog_currencies_enabled', 'no' ) ) {
				add_filter( 'alg_wc_cog_order_total_for_pecentage_fees', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_order_line_total', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_order_extra_cost_from_meta', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_order_shipping_total', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_order_total_fees', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_order_total_taxes', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_currencies_costs_percentage_total', array( $this, 'convert_currency' ), 10, 2 );
				add_filter( 'alg_wc_cog_shipping_total_for_pecentage_fees', array( $this, 'convert_currency' ), 10, 2 );
				if ( 'yes' === get_option( 'alg_wc_cog_currencies_wmc', 'no' ) ) {
					add_filter( 'alg_wc_cog_currency_rate', array( $this, 'wmc_get_currency_rate' ), 10, 3 );
				}
				add_filter( 'wmc_admin_reports', array( $this, 'wmc_add_cog_report' ), 10, 2 );
			}

			// Bkg Process.
			$this->init_bkg_process();
		}

		/**
		 * wmc_add_cog_report.
		 *
		 * @version 2.2.0
		 * @since   2.2.0
		 * @see     https://wordpress.org/plugins/woo-multi-currency/
		 */
		function wmc_add_cog_report( $value, $report ) {
			return ( 'alg_cost_of_goods' === $report ? true : $value );
		}

		/**
		 * wmc_get_currency_rate.
		 *
		 * @version 3.0.2
		 * @since   2.2.0
		 * @see     https://wordpress.org/plugins/woo-multi-currency/
		 */
		function wmc_get_currency_rate( $rate, $order, $wc_currency ) {
			$order_currency = $order->get_currency();
			if ( ( ! is_admin() || is_ajax() ) && function_exists( 'wmc_get_exchange_rate' ) ) {
				// From function
				return wmc_get_exchange_rate( $order_currency );
			} else {
				// From order meta
				$wmc_order_info = $order->get_meta( 'wmc_order_info', true );
				return ( ! empty( $wmc_order_info[ $order_currency ]['rate'] ) && is_numeric( $wmc_order_info[ $order_currency ]['rate'] ) ? $wmc_order_info[ $order_currency ]['rate'] : $rate );
			}
		}

		/**
		 * get_wc_currency.
		 *
		 * @version 2.3.0
		 * @since   2.2.0
		 */
		function get_wc_currency() {
			if ( ! isset( $this->wc_currency ) ) {
				$this->wc_currency = $this->core->get_default_shop_currency();
			}
			return $this->wc_currency;
		}

		/**
		 * get_currencies.
		 *
		 * @version 2.2.0
		 * @since   2.2.0
		 */
		function get_currencies() {
			if ( ! isset( $this->currencies ) ) {
				$this->currencies = get_option( 'alg_wc_cog_currencies', array() );
			}
			return $this->currencies;
		}

		/**
		 * get_currencies_rates.
		 *
		 * @version 2.2.0
		 * @since   2.2.0
		 */
		function get_currencies_rates() {
			if ( ! isset( $this->currencies_rates ) ) {
				$this->currencies_rates = get_option( 'alg_wc_cog_currencies_rates', array() );
			}
			return $this->currencies_rates;
		}

		/**
		 * convert_currency.
		 *
		 * @version 2.2.0
		 * @since   2.2.0
		 * @todo    [now] get rates from "Booster for WooCommerce" plugin
		 * @todo    [next] add "From order meta": a) same key for all currencies, and b) different meta key per currency
		 * @todo    [next] add automatic exchange rates (e.g. ECB)
		 * @todo    [maybe] check for `is_a( $order, 'WC_Order' )`?
		 */
		function convert_currency( $value, $order ) {
			if ( $value && ( $order_currency = $order->get_currency() ) != ( $wc_currency = $this->get_wc_currency() ) ) {
				// Filter
				if ( ( $rate = apply_filters( 'alg_wc_cog_currency_rate', 0, $order, $wc_currency ) ) ) {
					return ( $value / $rate );
				}
				// From options
				$currencies = $this->get_currencies();
				if ( in_array( $order_currency, $currencies ) ) {
					$rates = $this->get_currencies_rates();
					$rate  = apply_filters( 'alg_wc_cog_currency_rate_option', ( isset( $rates[ $wc_currency . $order_currency ] ) ? $rates[ $wc_currency . $order_currency ] : 0 ),
						$order, $wc_currency );
					if ( $rate ) {
						return ( $value / $rate );
					}
				}
			}
			return $value;
		}

		/**
		 * maybe_add_date_params_to_query.
		 *
		 * @version 2.9.2
		 * @since   2.9.2
		 *
		 * @param $query_args
		 *
		 * @return mixed
		 */
		function maybe_add_date_params_to_query( $query_args ) {
			$before_option_name = 'alg_wc_cog_recalculate_orders_cost_and_profit_before';
			$after_option_name  = 'alg_wc_cog_recalculate_orders_cost_and_profit_after';
			$before             = isset( $_POST[ $before_option_name ] ) ? $_POST[ $before_option_name ] : '';
			$after              = isset( $_POST[ $after_option_name ] ) ? $_POST[ $after_option_name ] : '';
			$order_date_status  = get_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_date_type', 'date_created' );
			if ( ! empty( $after ) && empty( $before ) ) {
				$query_args[ $order_date_status ] = '>' . ' ' . strtotime( $after );
			} elseif ( ! empty( $before ) && empty( $after ) ) {
				$query_args[ $order_date_status ] = '<' . ' ' . strtotime( $before );
			} elseif ( ! empty( $after ) && ! empty( $before ) ) {
				$query_args[ $order_date_status ] = strtotime( $after ) . '...' . strtotime( $before );
			}
			return $query_args;
		}

		/**
		 * recalculate_orders_cost_and_profit.
		 *
		 * @version 2.9.2
		 * @since   1.1.0
		 * @todo    [next] maybe skip "refunded" orders?
		 * @todo    [next] add to bulk actions
		 */
		function recalculate_orders_cost_and_profit( $current_section ) {
			if (
				'yes' === get_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_all',      'no' ) ||
				'yes' === get_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_no_costs', 'no' )
			) {
				if ( 'tools' === $current_section && current_user_can( 'manage_woocommerce' ) ) {
					if ( 0 != ( $memory_limit = get_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_memory_limit', 0 ) ) ) {
						ini_set( 'memory_limit', $memory_limit . 'M' );
					}
					$recalculate_for_orders_with_no_costs = get_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_no_costs', 'no' );
					$query_args = array( 'limit' => - 1, 'return' => 'ids' );
					$query_args = $this->maybe_add_date_params_to_query( $query_args );
					$orders = wc_get_orders( $query_args );
					if ( count( $orders ) < get_option( 'alg_wc_cog_bkg_process_min_amount', 100 ) ) {
						foreach ( $orders as $order_id ) {
							$this->core->orders->update_order_items_costs( array(
								'order_id'         => $order_id,
								'is_new_order'     => true,
								'is_no_costs_only' => ( 'yes' === $recalculate_for_orders_with_no_costs ),
							) );
						}
						$this->add_admin_notice( 'success', __( 'Orders cost and profit successfully recalculated.', 'cost-of-goods-for-woocommerce' ) );
					} else {
						$this->recalculate_orders_bkg_process->cancel_process();
						foreach ( $orders as $order_id ) {
							$this->recalculate_orders_bkg_process->push_to_queue( array( 'id' => $order_id, 'recalculate_for_orders_with_no_costs' => ( 'yes' === $recalculate_for_orders_with_no_costs ) ) );
						}
						$this->recalculate_orders_bkg_process->save()->dispatch();
						$this->add_admin_notice( 'success', __( 'Orders cost and profit recalculating via background processing. You should receive an e-mail when it\'s complete if the "Advanced > Background processing > Send email" option is enabled.', 'cost-of-goods-for-woocommerce' ) );
					}
				} else {
					$this->add_admin_notice( 'error',   __( 'Something went wrong...', 'cost-of-goods-for-woocommerce' ) );
				}
				update_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_all',      'no' );
				update_option( 'alg_wc_cog_recalculate_orders_cost_and_profit_no_costs', 'no' );
				wc_delete_shop_order_transients();
			}
		}

		/**
		 * add_admin_notice.
		 *
		 * @version 1.4.7
		 * @since   1.4.0
		 */
		function add_admin_notice( $type, $message ) {
			if ( method_exists( 'WC_Admin_Settings', 'add_message' ) ) {
				WC_Admin_Settings::add_message( $message );
			} else {
				$this->admin_notice = array( 'type' => $type, 'message' => $message );
				add_action( 'admin_notices', array( $this, 'print_admin_notice' ), PHP_INT_MAX );
			}
		}

		/**
		 * print_admin_notice.
		 *
		 * @version 1.4.0
		 * @since   1.4.0
		 */
		function print_admin_notice() {
			echo '<div class="notice notice-' . $this->admin_notice['type'] . ' is-dismissible"><p><strong>' . $this->admin_notice['message'] . '</strong></p></div>';
		}

		/**
		 * add_cog_reports.
		 *
		 * @version 1.4.0
		 * @since   1.4.0
		 */
		function add_cog_reports( $reports ) {
			if ( ! apply_filters( 'alg_wc_cog_create_report_validation', true ) ) {
				return $reports;
			}
			// Orders
			$orders_report = array(
				'title'       => __( 'Cost of goods', 'cost-of-goods-for-woocommerce' ),
				'description' => '',
				'hide_title'  => true,
				'callback'    => array( $this, 'get_cog_report' ),
			);
			if ( isset( $reports['orders']['reports'] ) ) {
				$orders_reports = array();
				foreach ( $reports['orders']['reports'] as $report_id => $report_data ) {
					$orders_reports[ $report_id ] = $report_data;
					if ( 'sales_by_date' === $report_id ) {
						$orders_reports['alg_cost_of_goods'] = $orders_report;
					}
				}
				$reports['orders']['reports'] = $orders_reports;
			} else {
				$reports['orders']['reports']['alg_cost_of_goods'] = $orders_report;
			}
			// Stock
			$reports['stock']['reports']['alg_cost_of_goods_stock'] = array(
				'title'       => __( 'Cost of goods', 'cost-of-goods-for-woocommerce' ),
				'description' => '',
				'hide_title'  => true,
				'callback'    => array( $this, 'get_cog_report_stock' ),
			);
			return $reports;
		}

		/**
		 * get_cog_report.
		 *
		 * @version 2.3.4
		 * @since   1.3.0
		 * @todo    [next] `order_status` and transients?
		 */
		function get_cog_report() {
			require_once( 'reports/class-wc-report-alg-cog.php' );
			$report = new WC_Report_Alg_Cost_Of_Goods();
			$report->order_status = get_option( 'alg_wc_cog_report_orders_order_status', array( 'completed', 'processing', 'on-hold' ) );
			if ( empty( $report->order_status ) ) {
				$report->order_status = array( 'completed', 'processing', 'on-hold' );
			}
			add_filter( 'woocommerce_reports_order_statuses', function ( $order_status ) use($report){
				if (
					! isset( $_GET['report'] )
					|| 'alg_cost_of_goods' !== $_GET['report']
				) {
					return $order_status;
				}
				return $report->order_status;
			} );
			$report->output_report();
		}

		/**
		 * get_cog_report_stock.
		 *
		 * @version 2.1.0
		 * @since   1.3.2
		 */
		function get_cog_report_stock() {
			require_once( 'reports/class-wc-report-alg-cog-stock.php' );
			$report = new WC_Report_Alg_Cost_Of_Goods_Stock();
			$report->output_report();
		}

		/**
		 * settings.
		 *
		 * @version 1.4.0
		 * @since   1.4.0
		 */
		function settings( $value, $type = '', $args = array() ) {
			if ( 'report' === $type ) {
				return sprintf( __( 'Cost of Goods reports are in %s and in %s.', 'cost-of-goods-for-woocommerce' ),
					'<a href="' . admin_url( 'admin.php?page=wc-reports&tab=orders&report=alg_cost_of_goods' ) . '">' .
					__( 'Reports > Orders > Cost of Goods', 'cost-of-goods-for-woocommerce' ) . '</a>',
					'<a href="' . admin_url( 'admin.php?page=wc-reports&tab=stock&report=alg_cost_of_goods_stock' ) . '">' .
					__( 'Reports > Stock > Cost of Goods', 'cost-of-goods-for-woocommerce' ) . '</a>'
				);
			}
			return '';
		}

		/**
		 * Schedule an action if it's not already scheduled.
		 *
		 * @version	2.4.3
		 * @since	2.4.3
		 */
		function currency_rate_cron_activation() {
			if ( 'yes' === get_option( 'alg_wc_cog_auto_currency_rates', 'no' ) ) {
				if ( ! wp_next_scheduled( 'alg_wc_cog_currency_rate_update' ) ) {
					$this->save_multicurrency_rate();
					wp_schedule_event( time(), 'daily', 'alg_wc_cog_currency_rate_update' );
				}
			} else {
				$timestamp = wp_next_scheduled( 'alg_wc_cog_currency_rate_update' );
				wp_unschedule_event( $timestamp, 'alg_wc_cog_currency_rate_update' );
				wp_clear_scheduled_hook( 'alg_wc_cog_currency_rate_update' );
			}
		}

		/**
		 * Multicurrency rate.
		 *
		 * @version	2.4.3
		 * @since	2.4.3
		 */
		function save_multicurrency_rate() {
			// All currencies "Multicurrency"
			$currencies      = $this->get_currencies();
			$currencies_rate = $this->get_currencies_rates();
			// Get Current Currency
			$wc_currency = $this->get_wc_currency();
			// Load currency rate
			$currency_rate = $this->load_currency_rates( $wc_currency );
			// New currency rate for update
			$new_currency_rate = array();
			if ( $currency_rate ) {
				foreach ( $currency_rate as $key => $value ) {
					if ( in_array( $key, $currencies ) ) {
						$key                       = $wc_currency . $key;
						$new_currency_rate[ $key ] = $value;
						if ( strtoupper( $wc_currency ) == strtoupper( $key ) ) {
							$new_currency_rate[ $key ] = "1";
						}
					}
				}
			}
			// Update currency rates
			if ( ! empty( $currencies_rate ) && ! empty( $new_currency_rate ) ) {
				$currencies_rate = array_merge( $currencies_rate, $new_currency_rate );
				update_option( 'alg_wc_cog_currencies_rates', $currencies_rate );
			}
		}

		/**
		 * Load currency rates from the exchangerate-api.com.
		 *
		 * @version	2.4.3
		 * @since	2.4.3
		 */
		function load_currency_rates( $base_currency ) {
			// Fetching JSON
			$req_url       = 'https://open.er-api.com/v6/latest/' . $base_currency;
			$response_json = file_get_contents( $req_url );
			$rate_data     = array();
			// Continuing if we got a result
			if ( false !== $response_json ) {
				// Try/catch for json_decode operation
				try {
					// Decoding
					$response = json_decode( $response_json );
					// Check for success
					if ( $response->result === 'success' && ! empty( $response->rates ) ) {
						foreach ( $response->rates as $key => $value ) {
							$rate_data[ $key ] = $value;
						}
					}
				} catch ( Exception $e ) {
					error_log( $e->getMessage() );
				}
			}
			return $rate_data;
		}



	}

endif;

return new Alg_WC_Cost_of_Goods_Pro();
